#import <Cocoa/Cocoa.h>

#import "XeeController.h"

#define XeeNoMode 0
#define XeeMoveMode 1
#define XeeCopyMode 2



@class XeeImage,XeeView,XeeCollisionPanel,XeeRenamePanel;



@interface XeeDirectoryController:XeeController
{
	NSString *directory;
	NSMutableArray *dircontents;

	NSRecursiveLock *loadlock;
	BOOL loader_running;
	BOOL exiting;

	XeeImage *previmage,*nextimage;
	int previndex,currindex,nextindex;

	XeeImage *loadingimage;

	int *randomlist;

	int drawer_mode,pending_mode;

    IBOutlet NSDrawer *drawer;
    IBOutlet NSTextField *drawertext;
	IBOutlet NSTableView *table;
	IBOutlet XeeCollisionPanel *collisionpanel;
	IBOutlet XeeRenamePanel *renamepanel;
}

-(id)init;
-(void)dealloc;
-(void)windowWillClose:(NSNotification *)notification;
-(NSRect)availableScreenSpace;
-(NSSize)minViewSize;

-(BOOL)loadImage:(NSString *)filename;

-(void)displayImage:(int)index next:(int)next;
-(XeeImage *)imageAtIndex:(int)index;
-(void)setCurrentImage:(XeeImage *)image index:(int)index;
-(void)setPreviousImage:(XeeImage *)image index:(int)index;
-(void)setNextImage:(XeeImage *)image index:(int)index;

-(void)reloadImage;

-(void)imageLoader:(id)nothing;
-(void)launchLoader;

-(void)updateStatusBar;

-(void)sortFiles;
-(int)findFile:(NSString *)filename;
-(int)findNextFile:(NSString *)filename;
-(void)removeFile:(NSString *)filename;
-(void)insertFile:(NSString *)filename;

-(NSDrawer *)drawer;
-(NSString *)directory;
-(NSString *)currentFilename;

-(IBAction)skipNext:(id)obj;
-(IBAction)skipPrev:(id)obj;
-(IBAction)skipFirst:(id)obj;
-(IBAction)skipLast:(id)obj;
-(void)skipToFile:(NSString *)filename;
-(void)skip:(int)step;

-(IBAction)skipRandom:(id)sender;
-(IBAction)skipRandomPrev:(id)sender;
-(void)buildRandomList;

-(IBAction)revealInFinder:(id)sender;

-(IBAction)renameFileFromMenu:(id)sender;
-(void)renameFile:(NSString *)filename to:(NSString *)newname;

-(IBAction)deleteFileFromMenu:(id)sender;
-(IBAction)askAndDelete:(id)sender;
-(void)deleteAlertEnd:(NSAlert *)alert returnCode:(int)res contextInfo:(NSString *)filename;
-(void)deleteFile:(NSString *)filename;

-(IBAction)moveFile:(id)sender;
-(IBAction)copyFile:(id)sender;
-(void)triggerDrawer:(int)mode;
-(void)drawerDidClose:(NSNotification *)notification;
-(void)destinationListClick:(id)sender;
-(void)destinationPanelEnd:(NSOpenPanel *)panel returnCode:(int)res contextInfo:(NSString *)filename;
-(void)attemptToTransferFile:(NSString *)filename to:(NSString *)destination mode:(int)mode;
-(void)transferFile:(NSString *)filename to:(NSString *)destination mode:(int)mode;

-(IBAction)jpegAutoRotate:(id)sender;
-(IBAction)jpegRotateCW:(id)sender;
-(IBAction)jpegRotateCCW:(id)sender;
-(IBAction)jpegRotate180:(id)sender;
-(IBAction)jpegFlipHorizontal:(id)sender;
-(IBAction)jpegFlipVertical:(id)sender;

-(void)jpegTransform:(NSString *)option;

-(IBAction)launchAppFromMenu:(id)sender;
-(IBAction)launchDefaultEditor:(id)sender;

-(void)errorMessage:(NSString *)title text:(NSString *)text;

+(void)initialize;
+(NSArray *)controllers;

@end



@interface XeeDirectoryWindow:NSWindow
{
	IBOutlet XeeDirectoryController *controller;
}

-(void)keyDown:(NSEvent *)event;
-(void)scrollWheel:(NSEvent *)event;

@end
