#import "XeeImage.h"
#import "XeeStrings.h"



@implementation XeeImage

-(id)initWithFile:(NSString *)name firstBlock:(NSData *)block attributes:(NSDictionary *)attributes
{
	if(self=[super init])
	{
		format=nil;
		width=height=0;
		depth=nil;
		transparent=NO;
		back=nil;
		state=XeeImageStateFailed;
		stop=NO;

		updateselector=nil;
		updatetarget=nil;

		filename=[name retain];
		attrs=[attributes retain];
	}

	return self;
}

-(void)dealloc
{
	[filename release];
	[attrs release];
	[back release];
	[format release];
	[depth release];

	[super dealloc];
}



-(int)state { return state; }

-(void)setState:(int)newstate { state=newstate; }

-(BOOL)loaded { return state==XeeImageStateLoaded; }

-(BOOL)failed { return state==XeeImageStateFailed; }

-(BOOL)needsLoading { return state==XeeImageStateInit||state==XeeImageStateLoading; }



-(void)load {  }

-(void)stopLoading { stop=YES; }

-(BOOL)hasBeenStopped { return stop; }



-(int)frames { return 0; }

-(void)setFrame:(int)frame { }

-(int)frame { return 0; }



-(void)setUpdateAction:(SEL)selector target:(id)target
{
	updateselector=selector;
	updatetarget=target;
}

-(void)triggerUpdateAction
{
	if(updatetarget&&[updatetarget respondsToSelector:updateselector])
	[updatetarget performSelectorOnMainThread:updateselector withObject:self waitUntilDone:NO];
}

-(BOOL)animated { return NO; }

-(void)setAnimating:(BOOL)animating { }

-(void)setAnimatingDefault { }

-(BOOL)animating { return NO; }



//-(void)draw:(NSPoint)point { [self draw:point size:NSMakeSize(width,height)]; }

-(void)draw:(NSPoint)point size:(NSSize)size  bounds:(NSRect)bounds { }



-(NSString *)filename { return filename; }

-(int)filesize { return [attrs fileSize]; }

-(NSDate *)date { return [attrs fileModificationDate]; }

-(NSString *)format { return format; }

-(int)width { return width; }

-(int)height { return height; }

-(NSString *)depth { return depth; }

-(BOOL)transparent { return transparent; }

-(NSColor *)backgroundColor { return back; }

-(NSBitmapImageRep *)imageRep { return nil; }



-(NSString *)descriptiveFilesize
{
	int size=[self filesize];

	if(size<10000) return [NSString stringWithFormat:@"%d B",size];
	else if(size<102400) return [NSString stringWithFormat:@"%.2f kB",((float)size/1024.0)];
	else if(size<1024000) return [NSString stringWithFormat:@"%.1f kB",((float)size/1024.0)];
//	else if(size<10240000) return [NSString stringWithFormat:@"%d kB",size/1024];
	else return [NSString stringWithFormat:@"%d kB",size/1024];
}

-(NSString *)descriptiveDate
{
	return [[self date] description];
}



-(void)setFilename:(NSString *)name { [filename autorelease]; filename=[name retain]; }

-(void)setFormat:(NSString *)fmt { [format autorelease]; format=[fmt retain]; }

-(void)setDepth:(NSString *)d { [depth autorelease]; depth=[d retain]; }

-(void)setTransparent:(BOOL)trans { transparent=trans; }

-(void)setBackgroundColor:(NSColor *)col { [back autorelease]; back=[col retain]; }



-(void)setDepthIndexed:(int)colors { [self setDepth:[NSString stringWithFormat:S_INDEXEDDEPTH,colors]]; }

-(void)setDepthGrey:(int)greys { [self setDepth:[NSString stringWithFormat:S_GREYDEPTH,greys]]; }

-(void)setDepthRGB:(int)bits { [self setDepth:[NSString stringWithFormat:S_RGBDEPTH,bits]]; }

-(void)setDepthRGBA:(int)bits { [self setDepth:[NSString stringWithFormat:S_RGBADEPTH,bits]]; }

-(void)setDepthGreyAlpha:(int)bits { [self setDepth:[NSString stringWithFormat:S_GREYALPHADEPTH,bits]]; }



NSMutableArray *imageclasses=nil;

+(void)initialize
{
	if(!imageclasses) imageclasses=[[NSMutableArray alloc] initWithCapacity:8];
}

+(XeeImage *)imageForFilename:(NSString *)filename
{
	NSDictionary *attrs=[[NSFileManager defaultManager] fileAttributesAtPath:filename traverseLink:YES];
	if(!attrs) return nil;

	NSFileHandle *file=[NSFileHandle fileHandleForReadingAtPath:filename];
	if(!file) return nil;

	NSData *block=[file readDataOfLength:512];
	if(!block) return nil;

	[file closeFile];

	NSEnumerator *enumerator=[imageclasses objectEnumerator];
	Class class;

	while(class=[enumerator nextObject])
	{
		XeeImage *image=[[class alloc] initWithFile:filename firstBlock:block attributes:attrs];
		if(image)
		{
			if([image state]==XeeImageStateFailed) [image release];
			else return [image autorelease];
		}
	}

	return nil;
}

+(void)registerImageClass:(Class)class
{
	[imageclasses addObject:class];
}

+(NSArray *)fileTypes
{
	NSMutableArray *types=[NSMutableArray arrayWithCapacity:32];
	NSEnumerator *enumerator=[imageclasses objectEnumerator];
	Class class;

	while(class=[enumerator nextObject])
	{
		NSEnumerator *typeenum=[[class fileTypes] objectEnumerator];
		NSString *type;
		while(type=[typeenum nextObject]) if(![types containsObject:type]) [types addObject:type];
	}

	return types;
}

@end



@implementation XeeMultiImage

-(id)initWithFile:(NSString *)name firstBlock:(NSData *)block attributes:(NSDictionary *)attributes;
{
	if(self=[super initWithFile:name firstBlock:block attributes:attributes])
	{
		currindex=0;

		subimages=[[NSMutableArray alloc] initWithCapacity:4];

		if(subimages)
		{
			return self;
		}

		[self release];
	}

	return nil;
}

-(void)dealloc
{
	[subimages release];

	[super dealloc];
}

-(void)addSubImage:(XeeImage *)subimage
{
	[subimage setUpdateAction:@selector(subImageUpdate:) target:self];
	[subimage setFilename:filename];
	[subimages addObject:subimage];
}

-(void)addSubImages:(NSArray *)array
{
	NSEnumerator *enumerator=[array objectEnumerator];
	XeeImage *image;
	while(image=[enumerator nextObject]) [self addSubImage:image];
}

-(void)subImageUpdate:(XeeImage *)subimage
{
	if(!subimages||[subimages count]==0) return; // pretty unlikely
	if(subimage==[subimages objectAtIndex:currindex]) [self triggerUpdateAction];
}

-(XeeImage *)currentSubImage
{
	if([subimages count]==0) return nil;
	else return [subimages objectAtIndex:currindex];
}


-(int)frames { return [subimages count]; }

-(void)setFrame:(int)frame
{
	if([subimages count]==0) return;

	if(frame<0) frame=0;
	if(frame>=[subimages count]) frame=[subimages count]-1;
	currindex=frame;
}

-(int)frame
{
	return currindex;
}

-(void)draw:(NSPoint)point size:(NSSize)size bounds:(NSRect)bounds
{
	[[self currentSubImage] draw:point size:size bounds:bounds];
}

-(int)width
{
	XeeImage *curr=[self currentSubImage];
	if(curr) return [curr width];
	else return [super width];
}

-(int)height
{
	XeeImage *curr=[self currentSubImage];
	if(curr) return [curr height];
	else return [super height];
}

-(NSString *)depth
{
	XeeImage *curr=[self currentSubImage];
	if(curr) return [curr depth];
	else return [super depth];
}

-(BOOL)transparent
{
	XeeImage *curr=[self currentSubImage];
	if(curr) return [curr transparent];
	else return [super transparent];
}

-(NSColor *)backgroundColor
{
	XeeImage *curr=[self currentSubImage];
	if(curr) return [curr backgroundColor];
	else return [super backgroundColor];
}

-(NSBitmapImageRep *)imageRep
{
	XeeImage *curr=[self currentSubImage];
	if(curr) return [curr imageRep];
	else return [super imageRep];
}

@end
