#import "XeeImageSaver.h"
#import "XeeImage.h"
#import "XeeSimpleLayout.h"
#import "XeeStrings.h"

@implementation XeeSavePanel

+(void)runSavePanelForImage:(XeeImage *)image window:(NSWindow *)window
{
	XeeSavePanel *panel=[[[XeeSavePanel alloc] initWithImage:image] autorelease];

	[panel beginSheetForDirectory:nil file:nil modalForWindow:window modalDelegate:panel
	didEndSelector:@selector(savePanelDidEnd:returnCode:contextInfo:) contextInfo:nil];
}


-(id)initWithImage:(XeeImage *)img
{
	if(self=[super init])
	{
		image=[img retain];

		wasanimating=[image animating];
		[image setAnimating:NO];

		savers=[[XeeImageSaver saversForImage:image] retain];

		NSMutableArray *pages=[NSMutableArray arrayWithCapacity:[savers count]];
		NSMutableArray *names=[NSMutableArray arrayWithCapacity:[savers count]];
		NSEnumerator *enumerator=[savers objectEnumerator];
		XeeImageSaver *saver;
		int def=0;

		while(saver=[enumerator nextObject])
		{
			id control=[saver control];
			if(!control) control=[NSNull null];
			[pages addObject:control];
			[names addObject:[saver format]];
		}

		formats=[[[XeeSLPages alloc] initWithTitle:@"Format" pages:pages names:names defaultValue:def] autorelease];
		view=[[[XeeSimpleLayout alloc] initWithControl:formats] autorelease];
		[view setDelegate:self];

		NSView *superview=[[[NSView alloc] initWithFrame:[view frame]] autorelease];
		[superview addSubview:view];

		[self setAccessoryView:superview];
	}
	return self;
}

-(void)dealloc
{
	[image release];
	[savers release];
	[formats release];
	[view release];

	[super dealloc];
}

-(void)savePanelDidEnd:(NSSavePanel *)sheet returnCode:(int)res contextInfo:(void *)info
{
	if(res==NSOKButton)
	{
		NSString *filename=[self filename];

		int page=[formats value];
		if([(XeeImageSaver *)[savers objectAtIndex:page] save:filename])
		{
			NSApplication *app=[NSApplication sharedApplication];
			[[app delegate] application:app openFile:filename];
		}
		else
		{
			NSAlert *alert=[[[NSAlert alloc] init] autorelease];
			[alert setMessageText:S_SAVERRORTITLE];
			[alert setInformativeText:[NSString stringWithFormat:S_SAVERRORCONTENT,filename]];
			[alert addButtonWithTitle:S_OK];
			[alert runModal];
		}
	}

	[image setAnimating:wasanimating];
}

-(void)xeeSLUpdated:(XeeSimpleLayout *)alsoview
{
	NSSavePanel *panel=[view window];

	NSView *superview=[[[NSView alloc] initWithFrame:[view frame]] autorelease];
	[view retain];
	[view removeFromSuperview];
	[superview addSubview:view];
	[view release];

	[panel setAccessoryView:superview];
}

@end



@implementation XeeImageSaver

-(id)initWithImage:(XeeImage *)img
{
	if(self=[super init])
	{
		image=[img retain];
		control=nil;
	}
	return self;
}

-(void)dealloc
{
	[image release];
	[control release];
	[super dealloc];
}

-(NSString *)format { return nil; }

-(BOOL)save:(NSString *)filename { return NO; }

-(XeeSLControl *)control { return control; }

-(void)setControl:(XeeSLControl *)cont { [control autorelease]; control=[cont retain]; }




static NSMutableArray *savers=nil;

+(void)initialize
{
	if(!savers) savers=[[NSMutableArray alloc] initWithCapacity:8];
}

+(NSArray *)saversForImage:(XeeImage *)image;
{
	NSMutableArray *instances=[NSMutableArray arrayWithCapacity:[savers count]];
	NSEnumerator *enumerator=[savers objectEnumerator];
	Class class;

	while(class=[enumerator nextObject]) [instances addObject:[[[class alloc] initWithImage:image] autorelease]];

	return instances;
}

+(void)registerSaverClass:(Class)class
{
	[savers addObject:class];
}

@end



@implementation XeeNSImageSaver

-(id)initWithImage:(XeeImage *)img
{
	if(self=[super initWithImage:img])
	{
		type=0;
	}
	return self;
}

-(BOOL)save:(NSString *)filename
{
	NSData *data=[[image imageRep] representationUsingType:type properties:nil];

	if(data)
	{
		if([data writeToFile:filename atomically:YES])
		{
			return YES;
		}
	}

	return NO;
}

-(void)setFileType:(NSBitmapImageFileType)filetype { type=filetype; }

@end




@implementation XeeAlphaSaver

-(id)initWithImage:(XeeImage *)img
{
	if(self=[super initWithImage:img])
	{
		if([image transparent])
		{
			alpha=[[XeeSLSwitch switchWithTitle:nil label:S_SAVEALPHA defaultValue:YES] retain];
			[self setControl:alpha];
		}
		else alpha=nil;
	}
	return self;
}

-(void)dealloc
{
	[alpha release];
	[super dealloc];
}

-(BOOL)save:(NSString *)filename
{
	NSBitmapImageRep *rep=[image imageRep];
	BOOL prevalpha=[rep hasAlpha];

	if(alpha) [rep setAlpha:[alpha value]];

	BOOL res=[super save:filename];

	[rep setAlpha:prevalpha];

	return res;
}

@end




@implementation XeePNGSaver

-(id)initWithImage:(XeeImage *)img
{
	if(self=[super initWithImage:img])
	{
		//depth=[[XeeSLPopUp popUpWithTitle:@"Depth:" defaultValue:1 contents:@"256 colors",@"24 bit RGB",nil] retain];

		interlaced=[[XeeSLSwitch switchWithTitle:nil label:S_INTERLACED defaultValue:NO] retain];

		[self setControl:[XeeSLGroup groupWithControls:interlaced,[self control],nil]];
		[self setFileType:NSPNGFileType];
	}
	return self;
}

-(void)dealloc
{
	//[depth release];
	[interlaced release];
	[super dealloc];
}

-(NSString *)format { return @"PNG"; }

-(BOOL)save:(NSString *)filename
{
	[[image imageRep] setProperty:NSImageInterlaced withValue:[NSNumber numberWithBool:[interlaced value]]];

	return [super save:filename];
}

@end



@implementation XeeJP2Saver

-(id)initWithImage:(XeeImage *)img
{
	if(self=[super initWithImage:img])
	{
		quality=[[XeeSLSlider sliderWithTitle:S_QUALITY minLabel:S_QUALITYMIN maxLabel:S_QUALITYLOSSLESS min:0 max:1 defaultValue:0.8] retain];

		[self setControl:[XeeSLGroup groupWithControls:quality,[self control],nil]];
		[self setFileType:NSJPEG2000FileType];
	}
	return self;
}

-(void)dealloc
{
	[quality release];
	[super dealloc];
}

-(NSString *)format { return @"JPEG-2000"; }

-(BOOL)save:(NSString *)filename
{
	[[image imageRep] setProperty:NSImageCompressionFactor withValue:[NSNumber numberWithFloat:[quality value]]];

	return [super save:filename];
}

@end



@implementation XeeTIFFSaver

-(id)initWithImage:(XeeImage *)img
{
	if(self=[super initWithImage:img])
	{
		compression=[[XeeSLPopUp popUpWithTitle:S_TIFFCOMPRESSION defaultValue:1 contents:S_TIFFNONE,S_TIFFLZW,S_TIFFPACKBITS,nil] retain];

		[self setControl:[XeeSLGroup groupWithControls:compression,[self control],nil]];
		[self setFileType:NSTIFFFileType];
	}
	return self;
}

-(void)dealloc
{
	[compression release];
	[super dealloc];
}

-(NSString *)format { return @"TIFF"; }

-(BOOL)save:(NSString *)filename
{
	switch([compression value])
	{
		case 0: [[image imageRep] setProperty:NSImageCompressionFactor withValue:[NSNumber numberWithInt:NSTIFFCompressionNone]]; break;
		case 1: [[image imageRep] setProperty:NSImageCompressionFactor withValue:[NSNumber numberWithInt:NSTIFFCompressionLZW]]; break;
		case 2: [[image imageRep] setProperty:NSImageCompressionFactor withValue:[NSNumber numberWithInt:NSTIFFCompressionPackBits]]; break;
	}

	return [super save:filename];
}

@end



@implementation XeeBMPSaver

-(id)initWithImage:(XeeImage *)img
{
	if(self=[super initWithImage:img])
	{
		[self setFileType:NSBMPFileType];
	}
	return self;
}

-(NSString *)format { return @"BMP"; }

@end



@implementation XeeJPEGSaver

-(id)initWithImage:(XeeImage *)img
{
	if(self=[super initWithImage:img])
	{
		quality=[[XeeSLSlider sliderWithTitle:S_QUALITY minLabel:S_QUALITYMIN maxLabel:S_QUALITYMAX min:0 max:1 defaultValue:0.8] retain];

		[self setControl:quality];
		[self setFileType:NSJPEGFileType];
	}
	return self;
}

-(void)dealloc
{
	[quality release];
	[super dealloc];
}

-(NSString *)format { return @"JPEG"; }

-(BOOL)save:(NSString *)filename
{
	[[image imageRep] setProperty:NSImageCompressionFactor withValue:[NSNumber numberWithFloat:[quality value]]];

	return [super save:filename];
}

@end




@implementation XeeGIFSaver

-(id)initWithImage:(XeeImage *)img
{
	if(self=[super initWithImage:img])
	{
		if([image transparent])
		{
			transparency=[[XeeSLPopUp popUpWithTitle:S_TRANSPARENCY defaultValue:1 contents:S_TRANSNONE,S_TRANSNORMAL,S_TRANSDITHER,nil] retain];
			[self setControl:transparency];
		}
		else transparency=nil;

		[self setControl:transparency];
		[self setFileType:NSGIFFileType];
	}
	return self;
}

-(void)dealloc
{
	[transparency release];
	[super dealloc];
}

-(NSString *)format { return @"GIF"; }

-(BOOL)save:(NSString *)filename
{
	NSBitmapImageRep *rep=[image imageRep];
	BOOL prevalpha=[rep hasAlpha];

	if(transparency)
	switch([transparency value])
	{
		case 0:
			[rep setAlpha:NO];
		break;

		case 1:
			[[image imageRep] setProperty:NSImageDitherTransparency withValue:[NSNumber numberWithBool:NO]];
		break;

		case 2:
			[[image imageRep] setProperty:NSImageDitherTransparency withValue:[NSNumber numberWithBool:YES]];
		break;
	}

	BOOL res=[super save:filename];

	[rep setAlpha:prevalpha];

	return res;
}

@end
