#import "XeePCXLoader.h"



#define read_word(ptr) ((ptr)[0]|((ptr)[1]<<8))



@implementation XeePCXImage

-(id)initWithFile:(NSString *)name firstBlock:(NSData *)block attributes:(NSDictionary *)attributes
{
	if(self=[super initWithFile:name firstBlock:block attributes:attributes])
	{
		fh=NULL;
		header=[block retain];

		if([block length]>128)
		{
			pcx=(struct pcx_header *)[block bytes];

			if(pcx->manufacturer==10
			&&(pcx->version==0||pcx->version==2||pcx->version==3||pcx->version==4||pcx->version==5)
			&&(pcx->nplanes==3&&(pcx->bitsperpixel==8)||(pcx->nplanes==1||pcx->bitsperpixel==8)
			  ||(pcx->nplanes==4||pcx->bitsperpixel==1)||(pcx->nplanes==1||pcx->bitsperpixel==1)))
			{
				width=read_word(pcx->xmax)-read_word(pcx->xmin)+1;
				height=read_word(pcx->ymax)-read_word(pcx->ymin)+1;

				if(pcx->nplanes==3) [self setDepthRGB:24];
				else [self setDepthIndexed:1<<(pcx->bitsperpixel*pcx->nplanes)];

				int pixelsperline=(8*read_word(pcx->bytesperline))/pcx->bitsperpixel;

				if(pixelsperline>=width) // sanity check
				{
					[self setFormat:@"PCX"];

					current_line=0;

					[self setState:XeeImageStateInit];
					return self;
				}
			}
		}

		[self release];
	}

	return nil;
}

-(void)dealloc
{
	if(fh) fclose(fh);
	[header release];

	[super dealloc];
}

-(void)load
{
	if([self state]==XeeImageStateInit)
	{
		fh=fopen([filename fileSystemRepresentation],"rb");
		if(!fh)
		{
			[self setState:XeeImageStateFailed];
			return;
		}

		if(![self allocBitmapWithType:XeeBitmapTypeRGB24 width:width height:height])
		{
			[self setState:XeeImageStateFailed];
			return;
		}

		if(pcx->version==5&&pcx->bitsperpixel==8)
		{
			fseek(fh,-769,SEEK_END);

			if(fgetc(fh)!=12)
			{
				[self setState:XeeImageStateFailed];
				return;
			}

			fread(palette,768,1,fh);

			fseek(fh,128,SEEK_SET);
		}
		else if(pcx->version==3) // no palette
		{
			byte defaultpalette[48]={0,0,0,0,0,170,0,170,0,0,170,170,170,0,0,170,0,170,170,85,0,
			170,85,85,85,85,85,0,0,255,0,255,0,0,255,255,255,0,0,255,0,255,255,255,0,255,255,255};
			memcpy(defaultpalette,pcx->colormap,48);
		}
		else
		{
			memcpy(palette,pcx->colormap,48);
		}

		[self setState:XeeImageStateLoading];
	}

	int bytesperline=read_word(pcx->bytesperline);
	int totalbytes=pcx->nplanes*bytesperline;
	byte line[totalbytes];

	stop=NO;

	while(!stop)
	{
		int bytesread=0;

		while(bytesread<totalbytes)
		{
			byte b=fgetc(fh);
			if(b<192) line[bytesread++]=b;
			else
			{
				byte c=fgetc(fh);
				int count=b-192;
				if(bytesread+count>totalbytes) count=totalbytes-bytesread;
				while(count--) line[bytesread++]=c;
			}
		}

		byte *imagerow=data+current_line*bytesperrow;

		if(pcx->nplanes==3&&pcx->bitsperpixel==8)
		{
			for(int i=0;i<width;i++)
			{
				*imagerow++=line[i];
				*imagerow++=line[i+bytesperline];
				*imagerow++=line[i+bytesperline*2];
			}
		}
		else if(pcx->nplanes==1&&pcx->bitsperpixel==8)
		{
			for(int i=0;i<width;i++)
			{
				byte *paletteentry=palette+line[i]*3;
				*imagerow++=*paletteentry++;
				*imagerow++=*paletteentry++;
				*imagerow++=*paletteentry++;
			}
		}
		else if(pcx->nplanes==4&&pcx->bitsperpixel==1)
		{
			for(int i=0;i<width;i++)
			{
				int col=0;
				col|=((line[(i>>3)+bytesperline*0]>>((i&7)^7))&1)<<0;
				col|=((line[(i>>3)+bytesperline*1]>>((i&7)^7))&1)<<1;
				col|=((line[(i>>3)+bytesperline*2]>>((i&7)^7))&1)<<2;
				col|=((line[(i>>3)+bytesperline*3]>>((i&7)^7))&1)<<3;
				byte *paletteentry=palette+col*3;
				*imagerow++=*paletteentry++;
				*imagerow++=*paletteentry++;
				*imagerow++=*paletteentry++;
			}
		}
		else if(pcx->nplanes==1&&pcx->bitsperpixel==1)
		{
			for(int i=0;i<width;i++)
			{
				int col=(line[i>>3]>>((i&7)^7))&1;
				byte *paletteentry=palette+col*3;
				*imagerow++=*paletteentry++;
				*imagerow++=*paletteentry++;
				*imagerow++=*paletteentry++;
			}
		}

		current_line++;
		if(current_line>=height) break;
	}

	if(stop) return;

	fclose(fh);
	fh=NULL;

	[header release];
	header=nil;

	[self setState:XeeImageStateLoaded];
	[self triggerUpdateAction];
}

+(NSArray *)fileTypes
{
	return [NSArray arrayWithObjects:@"pcx",@"'PCX '",@"'PCXf'",nil];
}

@end
